/**
* Main namespace contains the files for the CocoonAds plugin. 
* 
* @module Kiwi
* @submodule Plugins
* @namespace Kiwi.Plugins
* @class CocoonAds
*/
Kiwi.Plugins.CocoonAds = {

	/**
	* The name of this plugin.
	* @property name
	* @type String
	* @default "CocoonAds"
	* @public
	*/
	name: "CocoonAds",

	/**
	* The version of this plugin.
	* @property version
	* @type String
	* @public
	*/
	version: "0.1.0",

	/**
	* Minimum version of Kiwi this plugin requires.
	* @property minimumKiwiVersion
	* @type String
	* @default '1.3.0'
	* @public
	*/
	minimumKiwiVersion: "1.3.0"

};

/**
* Registers this plugin with the Global Kiwi Plugins Manager if it is avaiable.
*/
Kiwi.PluginManager.register( Kiwi.Plugins.CocoonAds );


/**
* Executed when the game is initially created. Assigns the game an internal reference to this object
* @method create
* @param game {Kiwi.Game} 
* @public
*/
Kiwi.Plugins.CocoonAds.create = function( game ) {
    
	game.ads = this;

	this.game = game;

    return this;	
}

/**
* Main namespace contains the files for the CocoonAds plugin. 
* 
* @module Kiwi
* @submodule Plugins
* @namespace Kiwi.Plugins
* @class CocoonAds
*/

/**
* Executed when the game boots. Called Interally.
*
* @method boot
* @public
*/ 
Kiwi.Plugins.CocoonAds.boot = function() {

	/**
	* If this plugin is enabled or not. 
	*
	* Not being enabled would mean that the Cocoon library has not been included.
	* 
	* @property enabled
	* @type boolean
	* @readOnly
	* @public
	*/
	this.enabled = true;

	//Banner Information

	/**
	* If the banner is currently visible.
	* 
	* @property _bannerVisible
	* @type boolean
	* @default false
	* @private
	*/
	this._bannerVisible = false;

	/**
	* If the banner is ready to be displayed or not.
	* 
	* @property _bannerReady
	* @type boolean
	* @default false
	* @private
	*/
	this._bannerReady = false;

	/**
	* If the banner is to be displayed at the top of the screen.
	*
	* The banner will display at the bottom if this is set to true.
	* 
	* @property _bannerTop
	* @type boolean
	* @default false
	* @private
	*/
	this._bannerTop = false;

	/**
	* The dimensions of the banner.
	* 
	* @property _bannerDimensions
	* @type Object
	* @private
	*/
	this._bannerDimensions = {

		/**
		* The width of the banner
		* 
		* @property _bannerDimensions.width
		* @type Number
		* @private
		*/
		width: 0,

		/**
		* The height of the banner
		* 
		* @property _bannerDimensions.height
		* @type Number
		* @private
		*/
		height: 0
	};

	/**
	* Dispatches events when the banner is ready to be used.
	* 
	* @property onBannerReady
	* @type Kiwi.Signal
	* @public
	*/
	this.onBannerReady = new Kiwi.Signal();

	/**
	* Dispatches events when the banners visiblity changes.
	*
	* Callbacks attached are passed a boolean representing visiblity of the banner. 
	* 
	* @property onBannerChange
	* @type Kiwi.Signal
	* @public
	*/
	this.onBannerChange = new Kiwi.Signal();

	//FullScreen Information

	/**
	* If a fullscreen ad is ready to be played.
	* 
	* @property fullscreenReady
	* @type boolean
	* @default false
	* @readOnly
	* @public
	*/
	this.fullscreenReady = false;

	/**
	* If a fullscreen ad is currently visible
	* 
	* @property fullscreenVisible
	* @type boolean
	* @default false
	* @readOnly
	* @public
	*/
	this.fullscreenVisible = false;

	/**
	* If a fullscreen ads should always be downloaded, and so ready to be played.
	*
	* Setting this to `true` will mean that when ever an ad stops playing, a new one will attempt to be played.
	* 
	* @property keepFullscreenAdReady
	* @type boolean
	* @default false
	* @public
	*/
	this.keepFullscreenAdReady = false;

	/**
	* If Kiwi should be disabled when a fullscreen ad is visible. 
	* 
	* @property disableKiwiWhenRun
	* @type boolean
	* @default true
	* @public
	*/
	this.disableKiwiWhenRun = true;

	/**
	* Holds the default states of kiwi methods. Used with the `disableKiwiWhenRun` method. 
	* 
	* @property _kiwiInformationRun
	* @type Object
	* @private
	*/
	this._kiwiInformationWhenRun = {};

	/**
	* Dispatches events when a fullscreen ad is ready to be used.
	* 
	* @property onFullscreenReady
	* @type Kiwi.Signal
	* @public
	*/
	this.onFullscreenReady = new Kiwi.Signal();

	/**
	* Dispatches events when the banners visiblity changes.
	*
	* Callbacks attached are passed a boolean representing visiblity of the ad. 
	* 
	* @property onFullscreenChange
	* @type Kiwi.Signal
	* @public
	*/
	this.onFullscreenChange = new Kiwi.Signal();


	//Check for cocoonjs support
	if( typeof Cocoon === "undefined" || typeof Cocoon.Ad === "undefined" ) {
    	Kiwi.Log.warn('CocoonAd: Cocoon API could not be found. Disabling this plugin.', '#ad', '#cocoon', '#disabled');
		this.enabled = false;
		return;
	}

	
	this._addEvents();

}


/**
* Returns a boolean indicating if the plugin is enabled or not.
* 
* @method _checkSupport
* @return {Boolean}
* @private
*/
Kiwi.Plugins.CocoonAds._checkSupport = function() {
	return this.enabled;
}

/**
* Adds callbacks to the related Cocoon events.
* 
* @method _addEvents
* @private
*/
Kiwi.Plugins.CocoonAds._addEvents = function() {
	
	//Banner 

	Cocoon.Ad.banner.on( "ready", function(width, height) {
    	Kiwi.Log.log('CocoonAd: Banner now ready to use', '#ad', '#cocoon');
        this._bannerDimensions.width = width;
        this._bannerDimensions.height = height;

        this._bannerReady = true;

        this._setBannerPosition();

        this.onBannerReady.dispatch();
    }.bind(this) );

    Cocoon.Ad.banner.on( "shown", function() {
    	Kiwi.Log.log('CocoonAd: Banner now visible', '#ad', '#cocoon');
        this._bannerVisible = true;

        this.onBannerChange.dispatch( this._bannerVisible );
    }.bind(this) );

    Cocoon.Ad.banner.on( "hidden", function() {
    	Kiwi.Log.log('CocoonAd: Banner now hidden', '#ad', '#cocoon');
        this._bannerVisible = true;

        this.onBannerChange.dispatch( this._bannerVisible );
    }.bind(this) );

    //Fullscreen

	Cocoon.Ad.interstitial.on("ready", function() {
    	Kiwi.Log.log('CocoonAd: Fullscreen now ready to use', '#ad', '#cocoon');
		this.fullscreenReady = true;
		this.onFullscreenReady.dispatch();
     }.bind(this) );

     Cocoon.Ad.interstitial.on("shown", function() {
    	Kiwi.Log.log('CocoonAd: Fullscreen now shown', '#ad', '#cocoon');
		this.fullscreenVisible = true;
		this.onFullscreenChange.dispatch( this.fullscreenVisible );
     }.bind(this) );

     Cocoon.Ad.interstitial.on("hidden", function() {
    	Kiwi.Log.log('CocoonAd: Fullscreen now hidden', '#ad', '#cocoon');
		this.fullscreenReady = false;
		this.fullscreenVisible = false;

		this._enableKiwi();

		if( this.keepFullscreenAdReady ) {
			this.loadFullscreen();
		}

		this.onFullscreenChange.dispatch( this.fullscreenVisible );
     }.bind(this) );
}


/**
* Internal method to set the banners position.
* 
* @method _setBannerPosition
* @private
*/
Kiwi.Plugins.CocoonAds._setBannerPosition = function() {

	//If not supported, or the banner not ready then hide
	if( !this._checkSupport() || !this._bannerReady ) return;

	var rect = Cocoon.Ad.getRectangle();

    var dpr = window.devicePixelRatio;
    if ( this.bannerTop ) {
        rect.x = window.innerWidth * dpr/2 - rect.width/2;
        rect.y = 0;

    } else {
        rect.x = window.innerWidth * dpr/2 - rect.width/2;
        rect.y = window.innerHeight * dpr - rect.height;
    }

    Cocoon.Ad.setRectangle( rect );

}

/**
* If the banner is to be displayed at the top of the screen.
*
* The banner will display at the bottom if this is set to true.
* 
* @property bannerTop
* @type boolean
* @public
*/
Object.defineProperty(Kiwi.Plugins.CocoonAds, 'bannerTop', {
	get: function() {
		return this._bannerTop;
	},

	set: function(val) {
		this._bannerTop = val;
		this._setBannerPosition();
	}
});


/**
* If the banner is currently visible.
* 
* @property bannerVisible
* @type boolean
* @default false
* @public
*/
Object.defineProperty(Kiwi.Plugins.CocoonAds, 'bannerVisible', {

	get: function() {
		return this._bannerVisible;
	},

	set: function(val) {
		if( val ) {
			this.showBanner();
		} else {
			this.hideBanner();
		}
	}

});


/**
* Shows a banner ad.   
* 
* @method showBanner
* @param [force=false] {Boolean} If the state of banner should be ignored.
* @public
*/
Kiwi.Plugins.CocoonAds.showBanner = function( force ) {
	
	if( !this._checkSupport() ) return;

	//Already visible?
	if( this._bannerVisible && !force ) {
		//Skip
		return;
	}

    this._setBannerPosition();
	Cocoon.Ad.showBanner();

}


/**
* hideBanner a banner ad. 
* 
* @method hideBanner
* @param [force=false] {Boolean} If the state of banner should be ignored.
* @public
*/
Kiwi.Plugins.CocoonAds.hideBanner = function( force ) {

	if( !this._checkSupport() ) return;

	//Already visible?
	if( !this._bannerVisible && !force ) {
		//Skip
		return;
	}

	Cocoon.Ad.showBanner();
}


/**
* Loads a banner ad. 
* Passing a boolean indicates whether or not it should be displayed when it is loaded. 
* 
* @method loadBanner
* @param [showWhenLoaded=false] {Boolean} 
* @public
*/
Kiwi.Plugins.CocoonAds.loadBanner = function( showWhenLoaded ) {

	if( !this._checkSupport() ) return;

	if( showWhenLoaded ) {

		this.onBannerReady.addOnce( this.showBanner, this );

	}

	Cocoon.Ad.loadBanner();

}

///Full Screen 

/**
* Internal method to disable Kiwi functionality
* 
* @method _disableKiwi
* @private
*/
Kiwi.Plugins.CocoonAds._disableKiwi = function() {
	
	if( this.disableKiwiWhenRun ) {
		this._kiwiInformationWhenRun.muted = this.game.audio.mute;
		this.game.audio.mute = true;
		
		this.game.input.onUp.active = false;
		this.game.input.onDown.active = false;
		this.game.input.mouse.onUp.active = false;
		this.game.input.mouse.onDown.active = false;
		this.game.input.touch.onUp.active = false;
		this.game.input.touch.onDown.active = false;

	}

}


/**
* Internal method to re-enable Kiwi functionality
* 
* @method _enableKiwi
* @private
*/
Kiwi.Plugins.CocoonAds._enableKiwi = function() {
	
	if( this.disableKiwiWhenRun ) {
		this.game.audio.mute = this._kiwiInformationWhenRun.muted;

		this.game.input.onUp.active = true;
		this.game.input.onDown.active = true;
		this.game.input.mouse.onUp.active = true;
		this.game.input.mouse.onDown.active = true;
		this.game.input.touch.onUp.active = true;
		this.game.input.touch.onDown.active = true;
	}

}


/**
* Plays a fullscreen ad.  
* 
* @method playFullscreen
* @param [force=false] {boolean} If the method should be executed regardless of the internal state.
* @public
*/
Kiwi.Plugins.CocoonAds.playFullscreen = function( force ) {

	if( !this._checkSupport() || !this.fullscreenReady && !force ) return;

	this._disableKiwi();

	Cocoon.Ad.showInterstitial();

}


/**
* Loads a fullscreen ad.  
* 
* @method loadFullscreen
* @param [showWhenLoaded=false] {boolean} If the fullscreen ad should be displayed when it loads or not.
* @public
*/
Kiwi.Plugins.CocoonAds.loadFullscreen = function( showWhenLoaded ) {

	if( !this._checkSupport() ) return;

	if( showWhenLoaded ) {
		this.onFullscreenReady.addOnce( this.playFullscreen, this );
	}

	Cocoon.Ad.loadInterstitial();

}
