/**
* Main namespace contains the files for the Flurry Analytics plugin. 
* 
* @module Kiwi
* @submodule Plugins
* @namespace Kiwi.Plugins
* @class FlurryAnalytics
*/
Kiwi.Plugins.FlurryAnalytics = {

	/**
	* The name of this plugin.
	* @property name
	* @type String
	* @default "FlurryAnalytics"
	* @public
	*/
	name: "FlurryAnalytics",

	/**
	* The version of this plugin.
	* @property version
	* @type String
	* @public
	*/
	version: "0.1.0",

	/**
	* Minimum version of Kiwi this plugin requires.
	* @property minimumKiwiVersion
	* @type String
	* @default '1.3.0'
	* @public
	*/
	minimumKiwiVersion: "1.3.0",

	/**
	* The API key of your app on Flurry. 
	* Assign a value to this **before** the plugin boots have
	* the 'startSession' method automatically called Flurry.     
	* @property API_KEY
	* @type String
	* @public
	*/
	API_KEY: null

};

/**
* Registers this plugin with the Global Kiwi Plugins Manager if it is avaiable.
*/
Kiwi.PluginManager.register( Kiwi.Plugins.FlurryAnalytics );


/**
* Executed when the game is initially created. Creates a new FlurryAnalytics Manager on the game to be used. 
* @method create
* @param game {Kiwi.Game} 
* @public
*/
Kiwi.Plugins.FlurryAnalytics.create = function( game ) {

	if( window['FlurryAnalyticsPlugin'] ) {
		game.flurry = window['FlurryAnalyticsPlugin'];
	} else {
		game.flurry = new Kiwi.Plugins.FlurryAnalytics.Manager(  );
	}

	return game.flurry;
};

/**
* Namespace which contains classes used for Flurry Analytics functionality.
* 
* @module Plugins
* @submodule FlurryAnalytics
*/

/**
* Contains the functionality for interacting with Flurry in native Cocoon instances. 
* Users should not directly instantate this class. 
*
* For this to successfully work users also need to use the cooon-webview extension. 
*
* @namespace Kiwi.Plugins.FlurryAnalytics
* @class Cocoon 
* @constructor
* @param manager {Kiwi.Plugins.FlurryAnalytics.Manager} The core Flurry manager this SDK is attached to.
*/
Kiwi.Plugins.FlurryAnalytics.Cocoon = function( manager ) {

	Kiwi.Log.log(' Kiwi.Plugins.FlurryAnalytics: Using Cocoon version of the Flurry Analytics.', '#flurry');

	/**
	* The ID of this class to idenify it from the Web version.
	* @property id 
	* @type String
	* @default 'FLURRY_COCOON'
	* @final
	* @readOnly
	* @public
	*/
	this.id = Kiwi.Plugins.FlurryAnalytics.Cocoon.ID;

	/**
	* The core Flurry manager
	* @property manager
	* @type Kiwi.Plugins.FlurryAnalytics.Manager
	* @public
	*/
	this.manager = manager;

	/**
	* If the SDK has been loaded and so is ready to be used or not.
	* @property ready
	* @type Boolean
	* @readOnly
	* @public
	*/
	this.ready = false;

	/**
	* If the next `call` method should also attempt to reload the SDK.
	* Only used if the SDK has not already been successfully loaded.
	*  
	* @property retry
	* @type Boolean
	* @private
	*/
	this.retry = false;

	/**
	* A list of Flurry calls that need to made once the SDK has been loaded.
	* 
	* @property queue
	* @type Array
	* @private
	*/
	this.queue = [];

	if( !window.Cocoon ) {
		Kiwi.Log.error( ' Kiwi.Plugins.FlurryAnalytics: To use this plugin with Cocoon please include the Cocoon library.', '#flurry' );
		return;
	}

}

/**
* The ID of this class to idenify it from the Web version.
* @property ID 
* @type String
* @default 'FLURRY_COCOON'
* @final
* @readOnly
* @static
* @public
*/
Kiwi.Plugins.FlurryAnalytics.Cocoon.ID = 'FLURRY_COCOON';

/**
* Executed when the SDK has successfully loaded. 
* @method onload
* @private
*/
Kiwi.Plugins.FlurryAnalytics.Cocoon.prototype.onload = function() {

	Kiwi.Log.log( 'Kiwi.Plugins.FlurryAnalytics: Flurry SDK loaded.', '#flurry' );
	this.ready = true;
	this.manager._ready = true;
	this.manager.boot();
	this.sendCommands();
	
}

/**
* Executed when the SDK has was not successfully loaded. 
* @method loaderror
* @private
*/
Kiwi.Plugins.FlurryAnalytics.Cocoon.prototype.loaderror = function() {

	Kiwi.Log.log( 'Kiwi.Plugins.FlurryAnalytics: Flurry SDK could not be loaded.', '#flurry' );
	this.retry = true;
}

/**
* Attempts to reload the SDK. 
* Used when the SDK could not be loaded the first time.
* @method retryLoad
* @public
*/
Kiwi.Plugins.FlurryAnalytics.Cocoon.prototype.retryLoad = function() {
	if( !this.ready ) {
		this.retry = false;
		Cocoon.App.forwardAsync( 'window.loadFlurry()' );
	}
}

/**
* Adds a command to the queue. 
* Will then attempt to send the commands if SDK is ready.
* 
* @method addToQueue
* @param cmd {string}
* @private
*/
Kiwi.Plugins.FlurryAnalytics.Cocoon.prototype.addToQueue = function( cmd ) {
	this.queue.push( cmd );
	if( this.ready ) {
		this.sendCommands();
	}

	if( this.retry ) {
		this.retryLoad();
	}
}


/**
* Sends all the commands in the queue. The queue will be flushed.
* 
* @method sendCommands
* @private
*/
Kiwi.Plugins.FlurryAnalytics.Cocoon.prototype.sendCommands = function() {
	var cmd;
	while( cmd = this.queue.shift() ){
		this.sendCommand( cmd );
	}
}


/**
* Sends a single the command to the webview for it to be executed. 
* 
* @method sendCommand
* @param cmd {String} The command to execute
* @private
*/
Kiwi.Plugins.FlurryAnalytics.Cocoon.prototype.sendCommand = function( cmd ) {
	Cocoon.App.forwardAsync( 'FlurryAgent.' + cmd );
}


/**
* Use this method to execute a Flurry method. You will need to pass the method you want to execute as a string
* and any arguments you want to pass as an array (in the order you want them passed). 
*
* If the SDK is *not ready* as the the calls will be added to a queue and executed when it successfully loads. 
* 
* @method call
* @param method {String}
* @param arguments {Array}
* @public
*/
Kiwi.Plugins.FlurryAnalytics.Cocoon.prototype.call = function(method, args) {

	if( !Kiwi.Utils.Common.isArray(args) ) {
		args = [args];
	}

	var argsString = '';
	for( var i = 0; i < args.length; i++) {
		if(i > 0){
			argsString += ',';
		}
		argsString += JSON.stringify( args[i] );
	}

	this.addToQueue(  method + '(' + argsString + ')' );
	
}


/**
* Namespace which contains classes used for Flurry Analytics functionality.
* 
* @module Plugins
* @submodule FlurryAnalytics
*/

/**
* The core manager which gets attached to core game objects. 
* Users either need to pass their API key to this method via calling the 'init' method, 
* OR by assigning `Kiwi.Plugins.FlurryAnalytics.API_KEY` a value before creating their game. 
* 
* Once in-game they can then call Flurry methods by using the 'call' method.
*
* @namespace Kiwi.Plugins.FlurryAnalytics
* @class Manager 
* @constructor
*/
Kiwi.Plugins.FlurryAnalytics.Manager = function(  ) {

	/**
	* The API Key that for the App on Flurry this game is related to.
	*
	* This should not be directly set. Instead you should use either the `init` method 
	* OR set `Kiwi.Plugins.FlurryAnalytics.API_KEY` before the plugin is created.
	*
	* @property apiKey
	* @type String
	* @default null
	* @public
	*/
	this.apiKey = null;

	/**
	* If the Flurry SDK has been included and so is ready to be used or not.
	* 
	* Users do not need to worry about calling methods when it is *not ready* as any event
	* logged when not ready will be added to a queue and executed when it is. 
	* 
	* @property _ready
	* @type boolean
	* @default false
	* @private
	*/
	this._ready = false;

	/**
	* Links to the SDK manager we are going to be using.
	* 
	* This will either be `Kiwi.Plugins.FlurryAnalytics.Cocoon` or `Kiwi.Plugins.FlurryAnalytics.Web`
	* depending on if the native Cocoon object was detected or not.
	* 
	* @property _sdk
	* @private
	*/
	this._sdk = null;

	/**
	* If we are using the Cocoon version of Flurry Analytics or not.
	* 
	* @property cocoonJS
	* @type Boolean
	* @public
	*/
	this.cocoonJS = ( window.Cocoon && Cocoon.nativeAvailable );

	window['FlurryAnalyticsPlugin'] = this;

	if( this.cocoonJS ) {
		this._sdk = new Kiwi.Plugins.FlurryAnalytics.Cocoon( this );
	} else {
		this._sdk = new Kiwi.Plugins.FlurryAnalytics.Web( this );
	}

}

/**
* Executed when a game including this plugin boots. 
* Checks to see if a API_KEY was set on `Kiwi.Plugins.FlurryAnalytics.API_KEY` and starts the sessions.
* 
* This methods should **NOT** be called by users.
*
* @method boot
* @public
*/
Kiwi.Plugins.FlurryAnalytics.Manager.prototype.boot = function() {

	if( this._ready ) {
		return;
	}

	//Do we have an API Key?
	if( Kiwi.Plugins.FlurryAnalytics.API_KEY ) {
		this.init( Kiwi.Plugins.FlurryAnalytics.API_KEY );
	}


}

/**
* Starts a new Flurry session. 
*
* This will be called for you if you set a value on `Kiwi.Plugins.FlurryAnalytics.API_KEY`
* 
* @method init
* @param apiKey {Number} The API Key of your Flurry App.
* @public
*/
Kiwi.Plugins.FlurryAnalytics.Manager.prototype.init = function( apiKey ) {

	if( !apiKey ) {
		Kiwi.Log.error(' Kiwi.Plugins.FlurryAnalytics: You need to pass an API Key.', '#flurry');
		return;
	}

	this.apiKey = apiKey;
	Kiwi.Plugins.FlurryAnalytics.API_KEY = apiKey;

	//If we are not ready yet then wait...
	//Load methods will re-execute this method on load
	if( !this._ready ) {
		return;
	}

	//Start flurry session
	if( this._sdk.id === Kiwi.Plugins.FlurryAnalytics.Cocoon.ID ) {
		this._sdk.sendCommand( 'startSession(' + JSON.stringify( Kiwi.Plugins.FlurryAnalytics.API_KEY ) + ')' );
	} else {
		this._sdk.sendCommand( 'startSession', [ Kiwi.Plugins.FlurryAnalytics.API_KEY ] );
	}

}

/**
* If the Flurry SDK has been included and so is ready to be used or not.
* 
* Users do not need to worry about calling methods when it is *not ready* as any event
* logged when not ready will be added to a queue and executed when it is. 
* 
* @property ready
* @type boolean
* @readOnly
* @public
*/
Object.defineProperty(Kiwi.Plugins.FlurryAnalytics.Manager.prototype, 'ready', {
	get: function() {
		return this._ready;
	}
});

/**
* Executed when using the Cocoon version of this Plugin by the webview. 
* Tells the SDK manager that the app is ready to dispatch calls to the Flurry Plugin.
* 
* @method webviewReady
* @private
*/
Kiwi.Plugins.FlurryAnalytics.Manager.prototype.webviewReady = function() {
	if( this._sdk.id === Kiwi.Plugins.FlurryAnalytics.Cocoon.ID ) {
		this._sdk.onload();
	}
}


/**
* Executed when using the Cocoon version of this Plugin by the webview. 
* Tells the SDK manager that the app could not load Flurry from its external URL.
* The SDK manager will then attempt to 'reload' the SDK on the `call` method dispatch.  
* 
* @method webviewFlurryNotLoaded
* @private
*/
Kiwi.Plugins.FlurryAnalytics.Manager.prototype.webviewFlurryNotLoaded = function() {
	if( this._sdk.id === Kiwi.Plugins.FlurryAnalytics.Cocoon.ID ) {
		this._sdk.loaderror();
	}
}


/**
* Use this method to execute a Flurry method. You will need to pass the method you want to execute as a string
* and any arguments you want to pass as an array (in the order you want them passed). 
*
* If the SDK is *not ready* as the the calls will be added to a queue and executed when it successfully loads. 
* 
* @method call
* @param method {String}
* @param args {Array}
* @public
*/
Kiwi.Plugins.FlurryAnalytics.Manager.prototype.call = function( method, args ) {
	this._sdk.call( method, args );
}


/**
* Nice API calls
**/


/**
* Executes the 'setUserId' method on Flurry. 
* 
* @method setUserId
* @param userId {any}
* @public
*/
Kiwi.Plugins.FlurryAnalytics.Manager.prototype.setUserId = function( userId ) {
	this._sdk.call( 'setUserId', userId );
}

/**
* Executes the 'setLocation' method on Flurry. 
* 
* @method setLocation
* @param lat {any}
* @param long {any}
* @param acc {any}
* @public
*/
Kiwi.Plugins.FlurryAnalytics.Manager.prototype.setLocation = function( lat, long, acc ) {
	this._sdk.call( 'setLocation', [lat, long, acc] );
}


/**
* Executes the 'logEvent' method on Flurry. 
* 
* @method logEvent
* @param event {any} The event we are logging
* @param [params] {object} Any extra parameters that should be passed
* @param [timedEvent] {boolean} if a timed event should start
* @public
*/
Kiwi.Plugins.FlurryAnalytics.Manager.prototype.logEvent = function( event, params, timedEvent ) {

	var args = [event];

	if( params ) {
		args.push( params );
	}

	if( timedEvent ) {
		args.push( true );
	}

	this._sdk.call( 'logEvent', args );
}

/**
* Executes the 'endTimedEvent' method on Flurry. 
* 
* @method endTimedEvent
* @param event {any} The event we are logging
* @param [params] {object} Any extra parameters that should be passed
* @public
*/
Kiwi.Plugins.FlurryAnalytics.Manager.prototype.endTimedEvent = function( event, params ) {

	var args = [event];

	if( params ) {
		args.push( params );
	}

	this._sdk.call( 'endTimedEvent', args );
}

/**
* Executes the 'logError' method on Flurry. 
* 
* @method logError
* @param errorName {string} 
* @param [errorMessage] {string} 
* @param [lineNumber] {string} 
* @public
*/
Kiwi.Plugins.FlurryAnalytics.Manager.prototype.logError = function( errorName, errorMessage, lineNumber ) {

	var args = [errorName];

	if( errorMessage ) {
		args.push( errorMessage );
	}

	if( lineNumber ) {
		args.push( lineNumber );
	}

	this._sdk.call( 'logError', args );
}

/**
* Executes the 'setRequestInterval' method on Flurry. 
* 
* @method setRequestInterval
* @param time {number} The new request interval
* @public
*/
Kiwi.Plugins.FlurryAnalytics.Manager.prototype.setRequestInterval = function( time ) {
	this._sdk.call( 'setRequestInterval', time );
}


/**
* Executes the 'setAppVersion' method on Flurry. 
* 
* @method setAppVersion
* @param version {string} 
* @public
*/
Kiwi.Plugins.FlurryAnalytics.Manager.prototype.setAppVersion = function( version ) {
	this._sdk.call( 'setAppVersion', version );
}

/**
* Namespace which contains classes used for Flurry Analytics functionality.
* 
* @module Plugins
* @submodule FlurryAnalytics
*/

/**
* Contains the functionality for interacting with Flurry in regular Web instances. 
* Users should not directly instantate this class. 
*
* @namespace Kiwi.Plugins.FlurryAnalytics
* @class Web 
* @constructor
* @param manager {Kiwi.Plugins.FlurryAnalytics.Manager} The core Flurry manager this SDK is attached to.
*/
Kiwi.Plugins.FlurryAnalytics.Web = function( manager ) {

	Kiwi.Log.log(' Kiwi.Plugins.FlurryAnalytics: Using Web version of the Flurry Analytics.', '#flurry' );

	/**
	* The ID of this class to idenify it from the Cocoon version.
	* @property id 
	* @type String
	* @default 'FLURRY_WEB'
	* @final
	* @readOnly
	* @public
	*/
	this.id = Kiwi.Plugins.FlurryAnalytics.Web.ID;

	/**
	* The core Flurry manager
	* @property manager
	* @type Kiwi.Plugins.FlurryAnalytics.Manager
	* @public
	*/
	this.manager = manager;

	/**
	* If the SDK has been loaded and so is ready to be used or not.
	* @property ready
	* @type Boolean
	* @readOnly
	* @public
	*/
	this.ready = false;

	/**
	* A list of Flurry calls that need to made once the SDK has been loaded.
	* 
	* @property queue
	* @type Array
	* @private
	*/
	this.queue = [];

	/**
	* Links to the Flurry SDK loaded.
	* @property sdk
	* @public 
	*/ 
	this.sdk = null;
	
	var script = document.createElement('script');
	script.onload = this.onload.bind( this );

	script.src = "https://cdn.flurry.com/js/flurry.js";
	document.head.appendChild( script );

}


/**
* The ID of this class to idenify it from the Cocoon version.
* @property ID 
* @type String
* @default 'FLURRY_WEB'
* @final
* @readOnly
* @static
* @public
*/
Kiwi.Plugins.FlurryAnalytics.Web.ID = 'FLURRY_WEB';


/**
* Executed when the SDK has successfully loaded. 
* @method onload
* @private
*/
Kiwi.Plugins.FlurryAnalytics.Web.prototype.onload = function() {

	if( !window.FlurryAgent ) {
		Kiwi.Log.warn(' Kiwi.Plugins.FlurryAnalytics: Flurry SDK not found or could not be loaded.', '#flurry');
		return;
	}

	Kiwi.Log.log( 'Kiwi.Plugins.FlurryAnalytics: Flurry SDK loaded. Ready to dispatch events.', '#flurry' );

	this.sdk = window.FlurryAgent;
	this.ready = true;
	this.manager._ready = true;
	this.manager.boot();
	this.sendCommands();
}


/**
* Adds a command to the queue. 
* Will then attempt to send the commands if SDK is ready.
* 
* @method addToQueue
* @param cmd {string}
* @private
*/
Kiwi.Plugins.FlurryAnalytics.Web.prototype.addToQueue = function( m, a ) {
	this.queue.push( m, a );
	if( this.ready ) {
		this.sendCommands();
	}
}

/**
* Sends all the commands in the queue. The queue will be flushed.
* 
* @method sendCommands
* @private
*/
Kiwi.Plugins.FlurryAnalytics.Web.prototype.sendCommands = function() {
	var cmd;
	while( cmd = this.queue.shift() ){
		this.sendCommand( cmd, this.queue.shift() );
	}
}

/**
* Executes a command on the Flurry SDK\
* 
* @method sendCommand
* @param cmd {String} The command to execute
* @private
*/
Kiwi.Plugins.FlurryAnalytics.Web.prototype.sendCommand = function( method, args ) {
	this.sdk[ method ].apply( this.sdk, args );
}

/**
* Use this method to execute a Flurry method. You will need to pass the method you want to execute as a string
* and any arguments you want to pass as an array (in the order you want them passed). 
*
* If the SDK is *not ready* as the the calls will be added to a queue and executed when it successfully loads. 
* 
* @method call
* @param method {String}
* @param arguments {Array}
* @public
*/
Kiwi.Plugins.FlurryAnalytics.Web.prototype.call = function(method, args) {

	if( !Kiwi.Utils.Common.isArray(args) ) {
		args = [args];
	}

	if( !this.ready ) {
		this.addToQueue( method, args );
	} else {
		this.sendCommand( method, args );
	}
}
