/**
* Namespace which contains the GamefrootLeaderboard plugin. 
* 
* @module Kiwi
* @submodule Kiwi.Plugins
* @namespace Kiwi.Plugins
* @class GamefrootLeaderboard
* @main
*/
Kiwi.Plugins.GamefrootLeaderboard = {
  
    /**
    * The name of this plugin.
    * @property name
    * @type String
    * @public
    */
    name: 'GamefrootLeaderboard',   

    /**
    * The version of this plugin in semver (semantic versioning) format
    * @property version
    * @type String
    * @public
    */
    version: '0.1.0',
    
    /**
    * The minimum version of Kiwi.js required to run this plugin in semver (semantic versioning) format
    * @property minimumKiwiVersion
    * @type String
    * @public
    */
    minimumKiwiVersion:'1.3.0',

    /**
    * The API key associated with your developer gamefroot account.
    * @property API_KEY
    * @type String
	* @public
    */
    API_KEY: null
};



/**
* Registers this plugin with the Global Kiwi Plugins Manager if it is avaiable.
*/
Kiwi.PluginManager.register( Kiwi.Plugins.GamefrootLeaderboard );


/**
* Executed when the game is initially created. Creates a new GamefrootLeaderboard Manager on the game to be used. 
* @method create
* @param game {Kiwi.Game} 
* @public
*/
Kiwi.Plugins.GamefrootLeaderboard.create = function( game ) {

	game.leaderboard = new Kiwi.Plugins.GamefrootLeaderboard.Manager( game );
	
	return game.leaderboard;
};

/**
* Namespace which contains classes used for Gamefroot leaderboards.
* 
* @module Plugins
* @submodule GamefrootLeaderboard
*/

/**
*
* @namespace Kiwi.Plugins.Manager
* @class Manager 
* @constructor
*/
Kiwi.Plugins.GamefrootLeaderboard.Manager = function( game ) {

	/**
	* The API Key that for your gamefroot developer account this game is related to.
	*
	* Checks to see if `Kiwi.Plugins.GamefrootLeaderboard.API_KEY` has a value, and then will contain that value.
	* Otherwise is set to null.
	*
	* @property apiKey
	* @type String
	* @default null
	* @public
	*/
	this.apiKey = null;

	/**
	* The game this leaderboard plugin is on.
	* 
	* @property game
	* @type Kiwi.Game
	* @public
	*/
	this.game = game;

	/**
	* The URL of the gamefroot server we will be posting our leaderboard information to.
	* 
	* @property _serverUrl
	* @type String
	* @default Kiwi.Plugins.GamefrootLeaderboard.Manager.LIVE_URL
	* @private
	*/
	this._serverUrl = Kiwi.Plugins.GamefrootLeaderboard.Manager.LIVE_URL;

	/**
	* Contains the last set of scores queried from the server.
	* 
	* @property scores
	* @type Array
	* @private
	*/
	this._scores = [];

	/**
	* The time at which the scores were last retrieved.
	* Does NOT use clocks, but instead `Date.now()`.
	* 
	* Updated each time the `get` method is called.
	* 
	* @property lastUpdated
	* @type Number
	* @default 0
	* @readOnly
	* @public
	*/
	this.lastUpdated = 0;

	if( Kiwi.Plugins.GamefrootLeaderboard.API_KEY ) {
		this.apiKey = Kiwi.Plugins.GamefrootLeaderboard.API_KEY;
	}

}

/**
* Returns a flag indicating if leaderboard functionality is ready or not.
* 
* @property ready
* @type Boolean
* @readOnly
* @public
*/
Object.defineProperty(Kiwi.Plugins.GamefrootLeaderboard.Manager.prototype, 'ready', {
	get: function() {
		return ( this.apiKey !== null && this.apiKey.length > 0 );
	}
});

/**
* 
* @method init
* @param apiKey {String} The API key associated with your account.
* @param live {Boolean} If the leaderboard should be using the live servers or not. Useful for testing leaderboard functionality without actually posting to live leadeboards.
* @public
*/
Kiwi.Plugins.GamefrootLeaderboard.Manager.prototype.init = function( apiKey, live ) {

	if( !apiKey ) {
		Kiwi.Log.error(" Kiwi.Plugins.GamefrootLeaderboard: You need to pass an api key in-order to call 'init'");
		return false;
	}
	this.apiKey = apiKey;
	Kiwi.Plugins.GamefrootLeaderboard.API_KEY = this.apiKey;

	if( typeof live == "undefined" ) {
		live = true;
	}

	if( !live ) {
		this._serverUrl = Kiwi.Plugins.GamefrootLeaderboard.Manager.STAGING_URL;
	} else {
		this._serverUrl = Kiwi.Plugins.GamefrootLeaderboard.Manager.LIVE_URL;
	}

	return true;
}


/**
* Makes a request to get the leaderboard scores. 
* 
* @method get
* @param [params] {Object}
* 	@param [params.order='score'] {String} The order the scores should be retrieved in from the server
*	@param [params.context] {Any} The context that callbacks attached should be called in.
*	@param [params.onComplete] {Function} A function which should be called when the process is completed. Regardless of success.
*	@param [params.onError] {Function} A function which should be called when the scores could not be retrieved.
*	@param [params.onSuccess] {Function} A function to be called when the scores were successfully retrieved.
* 	@param [params.sortedBy='score'] {String} Defines how the scores should be sorted when returned by the onSuccess callback. This sorting process is done CLIENTSIDE not SERVERSIDE.
* @public
*/
Kiwi.Plugins.GamefrootLeaderboard.Manager.prototype.get = function( params ) {

	if( !this.ready ) {
		Kiwi.Log.error(" Kiwi.Plugins.GamefrootLeaderboard: You need to call 'init' before you can get leaderboard information.");
		return;
	}

	params = params || {};
	params.order = params.order || 'score';

	return this._createNewGetRequest( 'leaderboards/get', { "order": params.order }, params );

}


/**
* Posts a score to the gamefroot server.
* 
* @method post
* @param params {Object}
*	@param params.score {Number} The score of this post.
*	@param params.user {String} The user (name) who got the score. 
*	@param [params.context] {Any} The context that callbacks attached should be called in.
*	@param [params.onComplete] {Function} A function which should be called when the process is completed. Regardless of success.
*	@param [params.onError] {Function} A function which should be called when the scores could not be retrieved.
*	@param [params.onSuccess] {Function} A function to be called when the scores were successfully retrieved.
* @return Kiwi.Files.DataFile
* @public
*/
Kiwi.Plugins.GamefrootLeaderboard.Manager.prototype.post = function( params ) {

	if( !this.ready ) {
		Kiwi.Log.error(" Kiwi.Plugins.GamefrootLeaderboard: You need to call 'init' before you can get leaderboard information.");
		return;
	}

	if( !params || !params.score || !params.user ) {
		Kiwi.Log.error(" Kiwi.Plugins.GamefrootLeaderboard: Have not passed the required parameters.");
		return;
	}

	var data = {
		'score': params.score,
		'user': params.user
	};

	return this._createNewPostRequest( 'leaderboards/set', data, params );
}


/**
* Makes a request to get the leaderboard scores using the `data` directory. 
* 
* @method getData
* @param [params] {Object}
* 	@param [params.order='score'] {String} The order the scores should be retrieved in from the server
* 	@param [params.offset=0] {Number} 
* 	@param [params.limit] {Number} 
* 	@param [params.user] {String} 
*	@param [params.context] {Any} The context that callbacks attached should be called in.
*	@param [params.onComplete] {Function} A function which should be called when the process is completed. Regardless of success.
*	@param [params.onError] {Function} A function which should be called when the scores could not be retrieved.
*	@param [params.onSuccess] {Function} A function to be called when the scores were successfully retrieved.
* 	@param [params.sortedBy='score'] {String} Defines how the scores should be sorted when returned by the onSuccess callback. This sorting process is done CLIENTSIDE not SERVERSIDE.
* @return Kiwi.Files.DataFile
* @public
*/
Kiwi.Plugins.GamefrootLeaderboard.Manager.prototype.getData = function( params ) {

	if( !this.ready ) {
		Kiwi.Log.error(" Kiwi.Plugins.GamefrootLeaderboard: You need to call 'init' before you can get leaderboard information.");
		return;
	}

	params = params || {};
	params.order = params.order || 'score';

	var data = {
		'order': params.order,
		'offset': params.offset || 0
	};

	if( params.limit ) {
		data.limit = params.limit; 
	}

	if( params.user ) {
		data.user = params.user;
	}

	return this._createNewGetRequest( 'leaderboards/data/get', data, params );
}


/**
* Posts a score to the gamefroot server.
* 
* @method postData
* @param params {Object}
*	@param params.score {Number} The score of this post.
*	@param params.user {String} The user (name) who got the score. 
*	@param [params.data={}] {Object} Any additional information.
*	@param [params.context] {Any} The context that callbacks attached should be called in.
*	@param [params.onComplete] {Function} A function which should be called when the process is completed. Regardless of success.
*	@param [params.onError] {Function} A function which should be called when the scores could not be retrieved.
*	@param [params.onSuccess] {Function} A function to be called when the scores were successfully retrieved.
* @return Kiwi.Files.Data.File
* @public
*/
Kiwi.Plugins.GamefrootLeaderboard.Manager.prototype.postData = function( params ) {

	if( !this.ready ) {
		Kiwi.Log.error(" Kiwi.Plugins.GamefrootLeaderboard: You need to call 'init' before you can get leaderboard information.");
		return;
	}

	if( !params || !params.score || !params.user ) {
		Kiwi.Log.error(" Kiwi.Plugins.GamefrootLeaderboard: Have not passed the required parameters.");
		return;
	}

	params.data = params.data || {};

	var data = {
		'score': params.score,
		'user': params.user
	};

	if( params.data ) {
		data.data = JSON.stringify( params.data );
	}

	return this._createNewPostRequest( 'leaderboards/data/set', data, params );
}

/**
* Posts a score to the gamefroot server.
* 
* @method updateData
* @param params {Object}
*	@param params.score {Number} The score of this post.
*	@param params.user {String} The user (name) who got the score. 
*	@param [params.data={}] {Object} Any additional information.
*	@param [params.context] {Any} The context that callbacks attached should be called in.
*	@param [params.onComplete] {Function} A function which should be called when the process is completed. Regardless of success.
*	@param [params.onError] {Function} A function which should be called when the scores could not be retrieved.
*	@param [params.onSuccess] {Function} A function to be called when the scores were successfully retrieved.
* @return Kiwi.Files.Data.File
* @public
*/
Kiwi.Plugins.GamefrootLeaderboard.Manager.prototype.updateData = function( params ) {

	if( !this.ready ) {
		Kiwi.Log.error(" Kiwi.Plugins.GamefrootLeaderboard: You need to call 'init' before you can get leaderboard information.");
		return;
	}

	if( !params || !params.score || !params.user ) {
		Kiwi.Log.error(" Kiwi.Plugins.GamefrootLeaderboard: Have not passed the required parameters.");
		return;
	}

	var data = {
		'score': params.score,
		'user': params.user
	};

	if( params.data ) {
		data.data = JSON.stringify( params.data );
	}

	return this._createNewPostRequest( 'leaderboards/data/update', data, params );

}

/**
* Returns the scores stored. 
* 
* @property scores
* @type Array
* @readOnly
* @public
*/
Object.defineProperty(Kiwi.Plugins.GamefrootLeaderboard.Manager.prototype, 'scores', {
	get: function(val) {
		return this._scores;
	}
});


/**
* Returns the scores. Another way of accessing the `scores`.
*
* You can define a method for the scores to be sorted by.
* See `Kiwi.Plugins.GamefrootLeaderboard.Manager.SORTED_BY` for a list of sorting methods.
* 
* @method getScores 
* @param [sortedBy='server'] {String} The order the scores should be sorted in before being returned. 
* @public 
*/
Kiwi.Plugins.GamefrootLeaderboard.Manager.prototype.getScores = function( sortedBy ) {

	sortedBy = sortedBy || 'server';

	switch( sortedBy.toLowerCase() ) {
		case Kiwi.Plugins.GamefrootLeaderboard.Manager.SORTED_BY.SCORE:
		case Kiwi.Plugins.GamefrootLeaderboard.Manager.SORTED_BY.HIGHEST:
			return this._scores.sort( function(a, b) {
				return b.score - a.score;
			} );
		case Kiwi.Plugins.GamefrootLeaderboard.Manager.SORTED_BY.LOWEST:
			return this._scores.sort( function(a, b) {
				return a.score - b.score;
			} );

		case Kiwi.Plugins.GamefrootLeaderboard.Manager.SORTED_BY.ALPHABETICAL: 
			return this._scores.sort( function(a, b) {
				if( a.user > b.user ) {
					return 1;
				} else if( a.user < b.user ) {
					return -1;
				} else {
					return 0;
				}

			} );
		case Kiwi.Plugins.GamefrootLeaderboard.Manager.SORTED_BY.REVERSE_ALPHABETICAL: 
		case Kiwi.Plugins.GamefrootLeaderboard.Manager.SORTED_BY.LACITEBAHPLA:
			return this._scores.sort( function(a, b) {
				if( a.user > b.user ) {
					return -1;
				} else if( a.user < b.user ) {
					return 1;
				} else {
					return 0;
				}
			} );

		case Kiwi.Plugins.GamefrootLeaderboard.Manager.SORTED_BY.SERVER:
		default:
			return this._scores;
	}

}


/**
* Holds the strings relating to the sorting methods that can be used.  
* 
* @property SORTED_BY
* @type Object
* @static
* @public
*/
Kiwi.Plugins.GamefrootLeaderboard.Manager.SORTED_BY = {

	/**
	* Sorts the leaderboards by their score.
	* 
	* @property SORTED_BY.SCORE
	* @type String
	* @default 'score'
	* @readOnly
	* @static
	* @public
	*/
	"SCORE": 'score',

	/**
	* Sorts the leaderboards by the username alphabetically.
	* 
	* @property SORTED_BY.ALPHABETICAL
	* @type String
	* @default 'alphabetical'
	* @readOnly
	* @static
	* @public
	*/
	"ALPHABETICAL": 'alphabetical',

	/**
	* Sorts the leaderboards by the username in reverse alphabetically order.
	* 
	* @property SORTED_BY.REVERSE_ALPHABETICAL
	* @type String
	* @default 'reverse-alphabetical'
	* @readOnly
	* @static
	* @public
	*/
	"REVERSE_ALPHABETICAL": 'reverse-alphabetical',

	/**
	* Sorts the leaderboards by the username in reverse alphabetically order.
	* 
	* @property SORTED_BY.LACITEBAHPLA
	* @type String
	* @default 'lacitebahpla'
	* @readOnly
	* @static
	* @public
	*/
	"LACITEBAHPLA": "lacitebahpla",

	/**
	* Sorts the leaderboards by the score with the lowest score being at the top.
	* 
	* @property SORTED_BY.LOWEST
	* @type String
	* @default 'lowest'
	* @readOnly
	* @static
	* @public
	*/
	"LOWEST": "lowest",

	/**
	* Sorts the leaderboards by the score with the highest being at the top.
	* 
	* @property SORTED_BY.HIGHEST
	* @type String
	* @default 'highest'
	* @readOnly
	* @static
	* @public
	*/
	"HIGHEST": "highest",

	/**
	* Doesn't score the scores. But instead returns the in the order the server returned them in.
	* 
	* @property SORTED_BY.SERVER
	* @type String
	* @default 'server'
	* @readOnly
	* @static
	* @public
	*/
	"SERVER": "server"
};


/**
* Updates the scores saved to those passed.
* 
* @method _setScores
* @param scores {Object}
* @private
*/
Kiwi.Plugins.GamefrootLeaderboard.Manager.prototype._setScores = function( scores ) {
	this._scores = scores;
	this.lastUpdated = Date.now();
}


/**
* Creates a new Kiwi DataFile which will query the gamefroot servers.
*  
* @method _createNewGetRequest
* @param endPoint {String} The API end point
* @param rawData {Object} Any additional information to be sent the server.
* @param params {Object} Parameters passed to the method whom called this one.
* @return Kiwi.Files.DataFile
* @private
*/
Kiwi.Plugins.GamefrootLeaderboard.Manager.prototype._createNewGetRequest = function( endPoint, rawData, params ) {

	rawData = rawData || {};
	rawData.game = this.game.stage.name;
	rawData['api-key'] = this.apiKey;


	var endUrl = '';
	for( var index in rawData ) {
		if( endUrl.length > 0 ) {
			endUrl += '&';
		} else {
			endUrl += '?';
		}
		endUrl += index + '=' + encodeURIComponent( rawData[ index ] ); 
	}


	var file = new Kiwi.Files.DataFile( this.game, {
		key: 'leaderboard-' + Date.now(), 
		url: this._serverUrl + endPoint + endUrl,
		type: Kiwi.Files.File.JSON
	} );
	file.parse = true;


	file.onComplete.add(function() {

		var error = false,
			msg = 'Could not successfully communicate with the server.';

		if( file.success ) {
			if( file.data.error ) {
				error = true;
				msg = file.data.msg;
			} 
		} else {
			error = true;
		}

		if( error ) {
			if( params.onComplete ) params.onComplete.call( params.context, false );
			if( params.onError ) params.onError.call( params.context, msg );
		} else {
			this._setScores( file.data );
			if( params.onComplete ) params.onComplete.call( params.context, true );
			if( params.onSuccess ) params.onSuccess.call( params.context, this.getScores( params.sortedBy ) );
		}

	}, this);

	this._loadFile( file );

	return file;

}


/**
* Creates a new Kiwi DataFile which will query the gamefroot servers.
*  
* @method _createNewPostRequest
* @param endPoint {String} The API end point
* @param rawData {Object} Any additional information to be sent the server.
* @param params {Object} Parameters passed to the method whom called this one.
* @return Kiwi.Files.DataFile
* @private
*/
Kiwi.Plugins.GamefrootLeaderboard.Manager.prototype._createNewPostRequest = function( endPoint, rawData, params ) {

	rawData = rawData || {};
	rawData.game = this.game.stage.name;
	rawData['api-key'] = this.apiKey;


	var file =  new Kiwi.Files.DataFile( this.game, {
		key: 'leaderboard-' + Date.now(), 
		url: this._serverUrl + endPoint,
		type: Kiwi.Files.File.JSON
	} );
	file.parse = true;

	var that = this;
	file.xhrLoader = function(method, requestType, timeout) {
		if( typeof requestType == 'undefined' ) {
			requestType = 'text';
		}

		that._postXhrLoader.call(file, requestType, rawData);
	};

	file.onComplete.add(function() {

		var error = false,
			msg = 'Could not successfully communicate with the server.';

		if( file.success ) {
			if( file.data.error ) {
				error = true;
				msg = file.data.msg;
			} 
		} else {
			error = true;
		}

		if( error ) {
			if( params.onComplete ) params.onComplete.call( params.context, false );
			if( params.onError ) params.onError.call( params.context, msg );
		} else {
			if( params.onComplete ) params.onComplete.call( params.context, true );
			if( params.onSuccess ) params.onSuccess.call( params.context );
		}

	}, this);

	this._loadFile( file );

	return file;

}

/**
* Starts the process of loading a file. 
*  
* @method _loadFile
* @param file {Kiwi.Files.DataFile} The file to start loading 
* @private
*/
Kiwi.Plugins.GamefrootLeaderboard.Manager.prototype._loadFile = function( file ) {
	this.game.loader.loadFile( file );
} 


/**
* The url of the leaderboard api on the gamefroot staging server. 
* 
* @property STAGING_URL
* @type String
* @default 'http://staging.gamefroot.com:8081/api/'
* @static
* @readOnly 
* @public
*/
Kiwi.Plugins.GamefrootLeaderboard.Manager.STAGING_URL = 'http://staging.gamefroot.com:8081/api/';


/**
* The url of the leaderboard api on the gamefroot live server. 
* 
* @property LIVE_URL
* @type String
* @default 'http://198.206.133.200:8081/api/'
* @static
* @readOnly 
* @public
*/
Kiwi.Plugins.GamefrootLeaderboard.Manager.LIVE_URL = 'http://198.206.133.200:8081/api/';



/**
* Function which is apart of the overriding the default XHR loader of Kiwi.Files.File 
* and instead replaces it with one which supports POST requests
* 
* @method _postXhrLoader
* @param [responseType='text'] {String}
* @param [data] {Number}
* @private
*/
Kiwi.Plugins.GamefrootLeaderboard.Manager.prototype._postXhrLoader = function(responseType, rawData) {

	//Stringify the data
    var data = '';
    for(var index in rawData) {
        if( data.length > 0) {
            data += '&';
        }
        data += index + '=' + rawData[index];
    }

    //XHR Request
	this._xhr = new XMLHttpRequest();
    this._xhr.open('POST', this.URL);
	this._xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");

	this._xhr.responseType = responseType;

    var _that = this;

	this._xhr.onload = function(event) { 
		_that.xhrOnLoad(event);
	};
	this._xhr.onerror = function(event) {
		_that.loadError(event);
	};
    this._xhr.onprogress = function(event) {
    	_that.xhrOnProgress(event);
    };
	this._xhr.onreadystatechange = function () {
        _that.readyState = _that._xhr.readyState;
	};
	this._xhr.onloadstart = function (event) {
        _that.timeStarted = event.timeStamp;
        _that.lastProgress = event.timeStamp;
	};
	this._xhr.ontimeout = function(event) {
        _that.hasTimedOut = true;
	};
    this._xhr.onloadend = function (event) {
        _that.xhrOnLoad(event);
    };

	this._xhr.send( data );
}